#ifdef fixed
#include <stdint.h>
typedef struct {
	int32_t x,y;
} coord;


int32_t qmul(int32_t a, int32_t b)
{
	return (int32_t) (((int64_t) a*b) >> 8);
}

int32_t qmul2(int32_t a, int32_t b)
{
	return (int32_t) (((int64_t) a*b) >> 24);
}

__inline int32_t qdiv(int32_t a, int32_t b)
{
	int64_t al, rl;
	al = (int64_t) a << 32;
	rl = al/b;
	return (int32_t) (rl >> 8);
}

void distance(coord p1, coord p2, coord p3, coord *p0)    //fixed
{
	int32_t u;    //u is in 8.24

	u = qdiv(qmul((p3.x - p1.x), (p2.x - p1.x)) + qmul((p3.y - p1.y), (p2.y - p1.y)), qmul((p2.x - p1.x), (p2.x - p1.x)) + qmul((p2.y - p1.y), (p2.y - p1.y)));
	if (u<0)
	{
		p0->x = p3.x - p1.x;
		p0->y = p3.y - p1.y;
	}
	else
		if (u>(1<<24))
		{
			p0->x = p3.x - p2.x;
			p0->y = p3.y - p2.y;
		}
		else
		{
			p0->x = p1.x + qmul2(u, (p2.x - p1.x));
			p0->x -= p3.x;
			p0->y = p1.y + qmul2(u, (p2.y - p1.y));
			p0->y -= p3.y;
		}

	if (p0->x<0)    //negativity check
		p0->x = -p0->x;
	if (p0->y<0)    //negativity check
		p0->y = -p0->y;
}
#else
typedef struct {
	float x,y;
} coord;
void distance(coord p1, coord p2, coord p3, coord *p0)    //float
{
	float u;

	u = ((p3.x - p1.x) * (p2.x - p1.x) + (p3.y - p1.y) * (p2.y - p1.y)) / ((p2.x - p1.x) * (p2.x - p1.x) + (p2.y - p1.y) * (p2.y - p1.y));
	if (u<0.0)
	{
		p0->x = p3.x - p1.x;
		p0->y = p3.y - p1.y;
	}
	else
		if (u>1.0)
		{
			p0->x = p3.x - p2.x;
			p0->y = p3.y - p2.y;
		}
		else
		{
			p0->x = p1.x + u * (p2.x - p1.x);
			p0->x -= p3.x;
			p0->y = p1.y + u * (p2.y - p1.y);
			p0->y -= p3.y;
		}

	if (p0->x<0)    //negativity check
		p0->x = -p0->x;
	if (p0->y<0)    //negativity check
		p0->y = -p0->y;
}
#endif

int main(void)
{
	coord p0, p1, p2, p3;
	unsigned ix, iy, i;

	for (i=0; i<8; i++) {
		p2.y = (120 + (i<<4))%240 << 8;
		p2.x = (160 + (i<<4))%320 << 8;
		p1.y = (0 + (i<<3))%240 << 8;
		p1.x = (0 + (i<<2))%320 << 8;
		for (iy=0; iy<240; iy++)
			for (ix=0; ix<320; ix++) {
				p3.x = ix<<8;
				p3.y = iy<<8;
				distance(p1, p2, p3, &p0);
			}
	}
	return 0;
}

